@@ -390,7 +390,7 @@ class Target(object):
     step = attr.ib(default=DEFAULT_STEP)
     target = attr.ib(default="")
     instant = attr.ib(validator=instance_of(bool), default=False)
-    datasource = attr.ib(default="")
+    datasource = attr.ib(default=None)
 
     def to_json_data(self):
         return {
@@ -544,12 +544,12 @@ def _balance_panels(panels):
 class GridPos(object):
     """GridPos describes the panel size and position in grid coordinates.
 
-        :param h: height of the panel, grid height units each represents
-            30 pixels
-        :param w: width of the panel 1-24 (the width of the dashboard
-            is divided into 24 columns)
-        :param x: x cordinate of the panel, in same unit as w
-        :param y: y cordinate of the panel, in same unit as h
+    :param h: height of the panel, grid height units each represents
+        30 pixels
+    :param w: width of the panel 1-24 (the width of the dashboard
+        is divided into 24 columns)
+    :param x: x cordinate of the panel, in same unit as w
+    :param y: y cordinate of the panel, in same unit as h
     """
 
     h = attr.ib()
@@ -653,10 +653,10 @@ class DashboardLink(object):
 class ExternalLink(object):
     """ExternalLink creates a top-level link attached to a dashboard.
 
-        :param url: the URL to link to
-        :param title: the text of the link
-        :param keepTime: if true, the URL params for the dashboard's
-            current time period are appended
+    :param url: the URL to link to
+    :param title: the text of the link
+    :param keepTime: if true, the URL params for the dashboard's
+        current time period are appended
     """
     uri = attr.ib()
     title = attr.ib()
@@ -679,24 +679,24 @@ class Template(object):
     """Template create a new 'variable' for the dashboard, defines the variable
     name, human name, query to fetch the values and the default value.
 
-        :param default: the default value for the variable
-        :param dataSource: where to fetch the values for the variable from
-        :param label: the variable's human label
-        :param name: the variable's name
-        :param query: the query users to fetch the valid values of the variable
-        :param refresh: Controls when to update values in the dropdown
-        :param allValue: specify a custom all value with regex,
-            globs or lucene syntax.
-        :param includeAll: Add a special All option whose value includes
-            all options.
-        :param regex: Regex to filter or capture specific parts of the names
-            return by your data source query.
-        :param multi: If enabled, the variable will support the selection of
-            multiple options at the same time.
-        :param type: The template type, can be one of: query (default),
-            interval, datasource, custom, constant, adhoc.
-        :param hide: Hide this variable in the dashboard, can be one of:
-            SHOW (default), HIDE_LABEL, HIDE_VARIABLE
+    :param default: the default value for the variable
+    :param dataSource: where to fetch the values for the variable from
+    :param label: the variable's human label
+    :param name: the variable's name
+    :param query: the query users to fetch the valid values of the variable
+    :param refresh: Controls when to update values in the dropdown
+    :param allValue: specify a custom all value with regex,
+        globs or lucene syntax.
+    :param includeAll: Add a special All option whose value includes
+        all options.
+    :param regex: Regex to filter or capture specific parts of the names
+        return by your data source query.
+    :param multi: If enabled, the variable will support the selection of
+        multiple options at the same time.
+    :param type: The template type, can be one of: query (default),
+        interval, datasource, custom, constant, adhoc.
+    :param hide: Hide this variable in the dashboard, can be one of:
+        SHOW (default), HIDE_LABEL, HIDE_VARIABLE
     """
 
     name = attr.ib()
@@ -804,6 +804,7 @@ DEFAULT_TIME = Time('now-1h', 'now')
 class TimePicker(object):
     """
     Time Picker
+
     :param refreshIntervals: dashboard auto-refresh interval options
     :param timeOptions: dashboard time range options
     :param hidden: hide the time picker from dashboard
@@ -1103,6 +1104,7 @@ class Dashboard(object):
 class Panel(object):
     """
     Generic panel for shared defaults
+
     :param cacheTimeout: metric query result cache ttl
     :param dataSource: Grafana datasource name
     :param description: optional panel description
@@ -1474,6 +1476,7 @@ class AlertList(object):
 
     :param dashboardTags: A list of tags (strings) for the panel.
     :param description: Panel description, supports markdown and links.
+    :param gridPos: describes the panel size and position in grid coordinates.
     :param id: panel id
     :param limit: Max number of alerts that can be displayed in the list.
     :param nameFilter: Show only alerts that contain nameFilter in their name.
@@ -1499,8 +1502,8 @@ class AlertList(object):
         validator=attr.validators.deep_iterable(
             member_validator=attr.validators.instance_of(str),
             iterable_validator=attr.validators.instance_of(list)))
-    description = attr.ib(default="")
-    gridPos = attr.ib(default=None)
+    description = attr.ib(default="", validator=instance_of(str))
+    gridPos = attr.ib(default=None, validator=instance_of(GridPos))
     id = attr.ib(default=None)
     limit = attr.ib(default=DEFAULT_LIMIT)
     links = attr.ib(
@@ -1508,6 +1511,7 @@ class AlertList(object):
         validator=attr.validators.deep_iterable(
             member_validator=attr.validators.instance_of(DataLink),
             iterable_validator=attr.validators.instance_of(list)))
+    nameFilter = attr.ib(default="", validator=instance_of(str))
     onlyAlertsOnDashboard = attr.ib(default=True, validator=instance_of(bool))
     show = attr.ib(default=ALERTLIST_SHOW_CURRENT)
     sortOrder = attr.ib(default=SORT_ASC, validator=in_([1, 2, 3]))
@@ -1618,6 +1622,7 @@ class Stat(Panel):
 class StatMapping(object):
     """
     Generates json structure for the value mapping for the Stat panel:
+
     :param text: Sting that will replace input value
     :param value: Value to be replaced
     :param startValue: When using a range, the start value of the range
@@ -1649,6 +1654,7 @@ class StatMapping(object):
 class StatValueMapping(object):
     """
     Generates json structure for the value mappings for the StatPanel:
+
     :param text: Sting that will replace input value
     :param value: Value to be replaced
     :param id: panel id
@@ -1666,6 +1672,7 @@ class StatValueMapping(object):
 class StatRangeMapping(object):
     """
     Generates json structure for the value mappings for the StatPanel:
+
     :param text: Sting that will replace input value
     :param startValue: When using a range, the start value of the range
     :param endValue: When using a range, the end value of the range
@@ -1717,8 +1724,6 @@ class SingleStat(Panel):
         Additional info can be found in docs:
         https://grafana.com/docs/grafana/latest/features/panels/singlestat/#value-to-text-mapping
     :param mappingTypes: the list of available mapping types for panel
-    :param maxDataPoints: maximum metric query results,
-        that will be used for rendering
     :param minSpan: minimum span number
     :param nullText: defines what to show if metric query result is undefined
     :param nullPointMode: defines how to render undefined values
@@ -1757,7 +1762,6 @@ class SingleStat(Panel):
             MAPPING_RANGE_TO_TEXT,
         ]),
     )
-    maxDataPoints = attr.ib(default=100)
     minSpan = attr.ib(default=None)
     nullText = attr.ib(default=None)
     nullPointMode = attr.ib(default='connected')
@@ -1789,7 +1793,6 @@ class SingleStat(Panel):
                 'hideTimeOverride': self.hideTimeOverride,
                 'mappingType': self.mappingType,
                 'mappingTypes': self.mappingTypes,
-                'maxDataPoints': self.maxDataPoints,
                 'minSpan': self.minSpan,
                 'nullPointMode': self.nullPointMode,
                 'nullText': self.nullText,
@@ -2091,8 +2094,6 @@ class BarGauge(Panel):
     :param limit: limit of number of values to show when not Calculating
     :param links: additional web links
     :param max: maximum value of the gauge
-    :param maxDataPoints: maximum metric query results,
-        that will be used for rendering
     :param min: minimum value of the gauge
     :param minSpan: minimum span number
     :param orientation: orientation of the bar gauge
@@ -2130,7 +2131,6 @@ class BarGauge(Panel):
     label = attr.ib(default=None)
     limit = attr.ib(default=None)
     max = attr.ib(default=100)
-    maxDataPoints = attr.ib(default=100)
     min = attr.ib(default=0)
     minSpan = attr.ib(default=None)
     orientation = attr.ib(
@@ -2157,7 +2157,6 @@ class BarGauge(Panel):
                 'cacheTimeout': self.cacheTimeout,
                 'hideTimeOverride': self.hideTimeOverride,
                 'interval': self.interval,
-                'maxDataPoints': self.maxDataPoints,
                 'minSpan': self.minSpan,
                 'options': {
                     'displayMode': self.displayMode,
@@ -2207,8 +2206,6 @@ class GaugePanel(Panel):
     :param limit: limit of number of values to show when not Calculating
     :param links: additional web links
     :param max: maximum value of the gauge
-    :param maxDataPoints: maximum metric query results,
-        that will be used for rendering
     :param min: minimum value of the gauge
     :param minSpan: minimum span number
     :param rangeMaps: the list of value to text mappings
@@ -2235,7 +2232,6 @@ class GaugePanel(Panel):
     label = attr.ib(default=None)
     limit = attr.ib(default=None)
     max = attr.ib(default=100)
-    maxDataPoints = attr.ib(default=100)
     min = attr.ib(default=0)
     minSpan = attr.ib(default=None)
     rangeMaps = attr.ib(default=attr.Factory(list))
@@ -2258,7 +2254,6 @@ class GaugePanel(Panel):
                 'cacheTimeout': self.cacheTimeout,
                 'hideTimeOverride': self.hideTimeOverride,
                 'interval': self.interval,
-                'maxDataPoints': self.maxDataPoints,
                 'minSpan': self.minSpan,
                 'options': {
                     'fieldOptions': {
@@ -2588,7 +2583,7 @@ class PieChart(Panel):
     :param id: panel id
     :param pieType: defines the shape of the pie chart (pie or donut)
     :param showLegend: defines if the legend should be shown
-    :param showLegend: defines if the legend should show values
+    :param showLegendValues: defines if the legend should show values
     :param legendType: defines where the legend position
     :param links: additional web links
     :param span: defines the number of spans that will be used for panel
@@ -2731,7 +2726,10 @@ class Threshold(object):
     :param op: EVAL_LT for less than or EVAL_GT for greater than to indicate what the threshold applies to.
     :param yaxis: Choose left or right for panels
 
-    Example:
+    Care must be taken in the order in which the Threshold objects are specified,
+    Grafana expects the value to increase.
+
+    Example::
         thresholds = [
             Threshold('green', 0, 0.0),
             Threshold('red', 1, 80.0)
